/*
** Copyright ? 2005 SerComm Corporation.
** All Rights Reserved. SerComm Corporation reserves the right to make changes
** to this document without notice. SerComm Corporation makes no warranty,
** representation or guarantee regarding the suitability of its products for
** any particular purpose. SerComm Corporation assumes no liability arising out
** of the application or use of any product or circuit. SerComm Corporation
** specifically disclaims any and all liability, including without limitation
** consequential or incidental damages; neither does it convey any license under
** its patent rights, nor the rights of others.
*/

#include <stdio.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <iptables.h>
#include <linux/types.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include <linux/netfilter/xt_TD.h>



/*
** For debugging
*/
#if 0
#define X_DBG printf
#else
#define X_DBG(format, args...)
#endif

static __BE32
dotted_to_ip(const CHAR *dotted)
{
	__BE32 addr;
	U_CHAR *addrp;
	CHAR *p, *q;
	U_INT onebyte;
	INT i;
	CHAR buf[DOT_IP_LEN];

	strncpy(buf, dotted, sizeof(buf) - 1);
	addrp=(U_CHAR *)&addr;
	p=buf;
	for(i=0; i<3; i++)
	{
		if((q=strchr(p, '.'))==NULL)
			return 0;
		*q='\0';
		if(string_to_number(p, 0, 255, &onebyte)==-1)
			return 0;
		addrp[i]=(U_CHAR)onebyte;
		p = q + 1;
	}
	if(string_to_number(p, 0, 255, &onebyte)==-1)
		return 0;
	addrp[3] = (U_CHAR) onebyte;

	return addr;
}

/*
** Function which prints out usage message.
*/
static struct option opts[]={
	{"sess_down_up", 1, NULL, P_SESS_DOWN_UP},
	{"wan_lan_wl", 1, NULL, P_WAN_LAN_WL},
	{"sessions", 1, NULL, P_SESSIONS},
	{"downstream", 1, NULL, P_DOWNSTREAM},
	{"upstream", 1, NULL, P_UPSTREAM},
	{"wan_mode", 1, NULL, P_WAN_MODE},
	{"remote_mac", 1, NULL, P_REMOTE_MAC},
	{"wan_if", 1, NULL, P_WAN_IF},
	{"wan_mac", 1, NULL, P_WAN_MAC},
	{"wan_ip", 1, NULL, P_WAN_IP},
	{"lan_if", 1, NULL, P_LAN_IF},
	{"lan_mac", 1, NULL, P_LAN_MAC},
	{"lan_ip", 1, NULL, P_LAN_IP},
	{"lan_subnet", 1, NULL, P_LAN_SUBNET},
	{"wl_if", 1, NULL, P_WL_IF},
	{"hw_nat", 1, NULL, P_HW_NAT},
	{"track_time", 1, NULL, P_TRACK_TIME},
	{"sleep_time", 1, NULL, P_SLEEP_TIME},
	{"long_sleep_time", 1, NULL, P_LONG_SLEEP_TIME},
	{"tcp_track_delay", 1, NULL, P_TCP_TRACK_DELAY},
	{"long_life_time", 1, NULL, P_LONG_LIFE_TIME},
	{"maybe_good", 1, NULL, P_MAYBE_GOOD_PORT_LIMIT},
	{"eliminate_noises", 1, NULL, P_ELIMINATE_NOISES_SCALE},
	{0}
};

static VOID
help(VOID)
{
	printf(
		"TD v%s options:\n"
		" --sessions <ip_conntrack_max> \n"
		" --downstream \n"
		" --upstream \n"
		" --wan_mode \n"
		" --wan_if \n"
		" --lan_if \n"
		" --wl_if \n"
		" --lan_subnet <ip>[</mask>]\n\n",
		IPTABLES_VERSION);
}

/*
** Initialize the target.
*/
static VOID
init(struct ipt_entry_target *t, U_INT *nfcache)
{
	return;
}

/*
** Function which parses command options; returns true if it ate an option.
*/
static INT
parse(INT c, CHAR **argv, INT invert, U_INT *flags,
      const struct ipt_entry *entry,
      struct ipt_entry_target **target)
{
	struct xt_td_target_info *info=(struct xt_td_target_info *)(*target)->data;
	INT arg_len=strlen(optarg), i=0;
	CHAR *p, *q, buf[EIGHTH_K];

	X_DBG("%s,%d: c=%d optarg=%s\n", __FUNCTION__, __LINE__, c, optarg);

	switch(c)
	{
		case P_SESS_DOWN_UP:
			strncpy(buf, optarg, sizeof(buf)-1);
			p=q=buf;
			while((p=strchr(q, '/'))!=NULL)
			{
				*p='\0';
				if(i==0)
					info->sessions=atoi(q);
				else if(i==1)
					info->downstream=atoi(q)<<7; /*Kbit to Byte*/
				i++;
				q=p+1;
			}
			if(i==2 && *q!='\0')
				info->upstream=atoi(q)<<7; /*Kbit to Byte*/
			X_DBG("%s,%d: sessions=%u down=%u up=%u\n", __FUNCTION__, __LINE__, info->sessions, info->downstream, info->upstream);
			break;
		case P_WAN_LAN_WL:
			strncpy(buf, optarg, sizeof(buf)-1);
			p=q=buf;
			while((p=strchr(q, '/'))!=NULL)
			{
				*p='\0';
				if(i==0)
					strcpy(info->wan_if, q);
				else if(i==1)
					strcpy(info->lan_if, q);
				i++;
				q=p+1;
			}
			if(i==2 && *q!='\0')
				strcpy(info->wl_if, q);
			X_DBG("%s,%d: wan_if=%s lan_if=%s wl_if=%s\n", __FUNCTION__, __LINE__, info->wan_if, info->lan_if, info->wl_if);
			break;
		case P_SESSIONS:
			info->sessions=atoi(optarg);
			break;
		case P_DOWNSTREAM:
			info->downstream=atoi(optarg)<<7; /*Kbit to Byte*/
			break;
		case P_UPSTREAM:
			info->upstream=atoi(optarg)<<7; /*Kbit to Byte*/
			break;
		case P_WAN_MODE:
			memset(info->wan_mode, 0, EIGHTH_K);
			if(arg_len>=EIGHTH_K)
				exit_error(PARAMETER_PROBLEM,
					"wan mode name `%s' must be shorter than EIGHTH_K(%d)",
					optarg, EIGHTH_K-1);
			strcpy(info->wan_mode, optarg);
			break;
		case P_REMOTE_MAC:
			break;
		case P_WAN_IF:
			memset(info->wan_if, 0, IFNAMSIZ);
			if(arg_len>=IFNAMSIZ)
				exit_error(PARAMETER_PROBLEM,
					"interface name `%s' must be shorter than IFNAMSIZ(%d)",
					optarg, IFNAMSIZ-1);
			strcpy(info->wan_if, optarg);
			X_DBG("%s,%d: wan_if<%s>\n", __FUNCTION__, __LINE__, info->wan_if);
			break;
		case P_WAN_MAC:
			break;
		case P_WAN_IP:
			break;
		case P_LAN_IF:
			memset(info->lan_if, 0, IFNAMSIZ);
			if(arg_len>=IFNAMSIZ)
				exit_error(PARAMETER_PROBLEM,
					"interface name `%s' must be shorter than IFNAMSIZ(%d)",
					optarg, IFNAMSIZ-1);
			strcpy(info->lan_if, optarg);
			X_DBG("%s,%d: lan_if<%s>\n", __FUNCTION__, __LINE__, info->lan_if);
			break;
		case P_LAN_MAC:
			break;
		case P_LAN_IP:
			break;
		case P_LAN_SUBNET:
			strncpy(buf, optarg, sizeof(buf)-1);
			if((p=strchr(buf, '/'))!=NULL)
			{
				*p='\0';
				p++;
				if(!p)
					info->lan_subnet[MASK_LAN]=0xFFFFFFFF;
				else
					info->lan_subnet[MASK_LAN]=dotted_to_ip(p);
			}
			info->lan_subnet[NET_LAN]=dotted_to_ip(buf) & info->lan_subnet[MASK_LAN];
			info->lan_subnet[MASK_LAN]=ntohl(info->lan_subnet[MASK_LAN]);
			info->lan_subnet[NET_LAN]=ntohl(info->lan_subnet[NET_LAN]);
			X_DBG("%s,%d: lan_subnet<%lx/%lx>\n", __FUNCTION__, __LINE__, info->lan_subnet[NET_LAN], info->lan_subnet[MASK_LAN]);
			break;
		case P_WL_IF:
			memset(info->wl_if, 0, IFNAMSIZ);
			if(arg_len>=IFNAMSIZ)
				exit_error(PARAMETER_PROBLEM,
					"interface name `%s' must be shorter than IFNAMSIZ(%d)",
					optarg, IFNAMSIZ-1);
			strcpy(info->wl_if, optarg);
			X_DBG("%s,%d: wl_if<%s>\n", __FUNCTION__, __LINE__, info->wl_if);
			break;
		case P_HW_NAT:
			if(!strcmp(optarg, "disable"))
				info->hwnat_en=0;
			else
				info->hwnat_en=1;
			break;
		case P_TRACK_TIME:
			info->track_time=atoi(optarg);
			break;
		case P_SLEEP_TIME:
			info->sleep_time=atoi(optarg);
			break;
		case P_LONG_SLEEP_TIME:
			info->long_sleep_time=atoi(optarg);
			break;
		case P_TCP_TRACK_DELAY:
			info->tcp_track_delay=atoi(optarg);
			break;
		case P_LONG_LIFE_TIME:
			info->long_life_time=atoi(optarg);
			break;
		case P_LARGE_TRAFFIC:
			info->large_traffic=atol(optarg);
			break;
		case P_MAYBE_GOOD_PORT_LIMIT:
			info->maybe_good=atoi(optarg);
			break;
		case P_ELIMINATE_NOISES_SCALE:
			info->eliminate_noises=atoi(optarg);
			break;
		default:
			exit_error(PARAMETER_PROBLEM, " sessions and downstream and upstream and inerfaces and lan_subnet must be set.\n");
	}

	return 1;
}

/*
** Final check
*/
static VOID final_check(U_INT flags)
{
	return;
}

/*
** Prints out the targinfo.
*/
static VOID
print(const struct ipt_ip *ip,
      const struct ipt_entry_target *target,
      INT numeric)
{
	//struct xt_td_target_info *info=(struct xt_td_target_info *)target->data;
	printf("traffic detection ");
}

/*
** Saves the union ipt_targinfo in parsable form to stdout.
*/
static VOID
save(const struct ipt_ip *ip, const struct ipt_entry_target *target)
{
	//struct xt_td_target_info *info=(struct xt_td_target_info *)target->data;
}

/*
** For registering
*/
static struct iptables_target traffic_detection = {
	.next		= NULL,
	.name		= "TD",
	.version	= IPTABLES_VERSION,
	.revision	= 0,
	.size		= IPT_ALIGN(sizeof(struct xt_td_target_info)),
	.userspacesize	= IPT_ALIGN(sizeof(struct xt_td_target_info)),
	.help		= &help,
	.init		= &init,
	.parse	= &parse,
	.final_check	= &final_check,
	.print	= &print,
	.save		= &save,
	.extra_opts	= opts
};

VOID _init(VOID)
{
	register_target(&traffic_detection);
}
